@@ -58,9 +58,9 @@ public:
     void setDefaultWindowAttributes (QWidget *pWidget);
     void setLongWork (QWidget *window, bool bSet = false);
 
-    QRect getStartingSize (QWidget *w);
-
+    QRect getStartingSize ();
     QString getAppDirectory ();
+    QString getMainQML();
 
     bool cipher(const QByteArray &byIn, QByteArray &byOut, bool bEncrypt);
 
