@@ -22,6 +22,8 @@ Contact: yuvraaj@gmail.com
 #ifndef TPOBSERVER_H
 #define TPOBSERVER_H
 
+#define USE_OLD_QT_TP4 0
+
 #include "global.h"
 #include <QtDBus>
 #include <TelepathyQt4/Types>
@@ -36,23 +38,30 @@ Contact: yuvraaj@gmail.com
 #include <TelepathyQt4/Channel>
 #include <TelepathyQt4/ChannelDispatchOperation>
 #include <TelepathyQt4/ChannelRequest>
+
+#if !USE_OLD_QT_TP4
+#include <TelepathyQt4/ChannelClassSpecList>
+#endif
+
 using namespace Tp;
 
 #include "IObserver.h"
 
+
 class TpObserver : public IObserver, public AbstractClientObserver
 {
     Q_OBJECT
 
 public:
-#if DESKTOP_OS || defined(MEEGO_HARMATTAN)
-    TpObserver (const ChannelClassSpecList &channelFilter,
-                      QObject              *parent = NULL);
-#else
+#if USE_OLD_QT_TP4
     TpObserver (const ChannelClassList &channelFilter,
                       QObject          *parent = NULL);
+#else
+    TpObserver (const ChannelClassSpecList &channelFilter,
+                      QObject              *parent = NULL);
 #endif
     void setId (int i);
+    QString name();
 
 protected:
     void startMonitoring (const QString &strC);
@@ -61,6 +70,7 @@ protected:
 private slots:
 
 protected:
+#if USE_OLD_QT_TP4
     void observeChannels(
             const MethodInvocationContextPtr<>  &context,
             const AccountPtr                    &account,
@@ -70,8 +80,7 @@ protected:
             const QList <ChannelRequestPtr>     &requestsSatisfied,
             const QVariantMap                   &observerInfo);
 
-#if DESKTOP_OS || defined(MEEGO_HARMATTAN)
-    // Linux and Meego have moved on to newer telepathy.
+#else    // Linux and Meego have moved on to newer telepathy.
     void observeChannels(
             const MethodInvocationContextPtr<>  &context,
             const AccountPtr                    &account,
